locals {
  events = split(",", var.events)
}

#s3 bucket
resource "aws_s3_bucket" "bucket" {
  bucket = var.bucket_name
  # This might not be ok in GOV, they want 'deletion prevention'
  force_destroy = true

  tags = var.tags
}

# s3 bucket policy
resource "aws_s3_bucket_policy" "allow_access_from_another_account" {
  bucket = aws_s3_bucket.bucket.id
  policy = file("bucket-policy.json")
}

#s3 acl
resource "aws_s3_bucket_public_access_block" "bucket_access_block" {
  bucket                  = aws_s3_bucket.bucket.id
  block_public_acls       = var.block_public_acls
  block_public_policy     = var.block_public_policy
  ignore_public_acls      = var.ignore_public_acls
  restrict_public_buckets = var.restrict_public_buckets
}

# Empty acl represents that ACL is disabled, as bucket ownership is set to BucketOwnerEnforced
resource "aws_s3_bucket_ownership_controls" "bucket_ownership" {
  count  = var.acl != "" ? 1 : 0
  bucket = aws_s3_bucket.bucket.id
  rule {
    object_ownership = "BucketOwnerPreferred"
  }
}

resource "aws_s3_bucket_acl" "acl" {
  count      = var.acl != "" ? 1 : 0
  depends_on = [aws_s3_bucket_ownership_controls.bucket_ownership[0], aws_s3_bucket_public_access_block.bucket_access_block]

  bucket = aws_s3_bucket.bucket.id
  acl    = var.acl
}

# s3 website config
resource "aws_s3_bucket_website_configuration" "website" {
  count  = var.s3_website_enabled ? 1 : 0
  bucket = aws_s3_bucket.bucket.id

  index_document {
    suffix = var.s3_website_index_document
  }
}

# s3 versioning
resource "aws_s3_bucket_versioning" "versioning" {
  count  = var.versioning_enabled ? 1 : 0
  bucket = aws_s3_bucket.bucket.id

  versioning_configuration {
    status = "Enabled"
  }
}

# s3 CORS config
resource "aws_s3_bucket_cors_configuration" "cors_rule" {
  count  = length(var.cors_rule) > 0 ? 1 : 0
  bucket = aws_s3_bucket.bucket.id

  dynamic "cors_rule" {
    for_each = var.cors_rule

    content {
      allowed_headers = lookup(cors_rule.value, "allowed_headers", [])
      allowed_methods = lookup(cors_rule.value, "allowed_methods", [])
      allowed_origins = lookup(cors_rule.value, "allowed_origins", [])
      expose_headers  = lookup(cors_rule.value, "expose_headers", [])
      max_age_seconds = lookup(cors_rule.value, "max_age_seconds", null)
    }
  }
}

# s3 logging config
resource "aws_s3_bucket_logging" "logging" {
  count  = var.logging_bucket_name == null ? 0 : 1
  bucket = aws_s3_bucket.bucket.id

  target_bucket = var.logging_bucket_name
  target_prefix = var.logging_prefix
}

# s3 lifecycle rule config
resource "aws_s3_bucket_lifecycle_configuration" "lifecycle_rules" {
  count = length(var.lifecycle_rule) > 0 ? 1 : 0
  # Must have versioning enabled
  depends_on = [aws_s3_bucket_versioning.versioning]
  bucket     = aws_s3_bucket.bucket.id

  dynamic "rule" {
    for_each = var.lifecycle_rule

    content {
      id     = lookup(rule.value, "id", null)
      status = lookup(rule.value, "enabled", false) ? "Enabled" : "Disabled"

      filter {
        prefix = lookup(rule.value, "prefix", null)

        and {
          tags = lookup(rule.value, "tags", {})
        }
      }

      abort_incomplete_multipart_upload {
        days_after_initiation = lookup(rule.value, "abort_incomplete_multipart_upload_days", null)
      }

      dynamic "expiration" {
        for_each = length(keys(lookup(rule.value, "expiration", {}))) == 0 ? [] : [lookup(rule.value, "expiration", {})]

        content {
          date                         = lookup(expiration.value, "date", null)
          days                         = lookup(expiration.value, "days", null)
          expired_object_delete_marker = lookup(expiration.value, "expired_object_delete_marker", null)
        }
      }

      dynamic "transition" {
        for_each = lookup(rule.value, "transition", [])

        content {
          date          = lookup(transition.value, "date", null)
          days          = lookup(transition.value, "days", null)
          storage_class = transition.value.storage_class
        }
      }

      # always use noncurrent_version_expiration in a lifecycle rule, default 1 day
      noncurrent_version_expiration {
        noncurrent_days = lookup(lookup(rule.value, "noncurrent_version_expiration", {}), "days", 1)
      }

      dynamic "noncurrent_version_transition" {
        for_each = lookup(rule.value, "noncurrent_version_transition", [])

        content {
          noncurrent_days = lookup(noncurrent_version_transition.value, "days", null)
          storage_class   = noncurrent_version_transition.value.storage_class
        }
      }
    }
  }
}

# s3 service side encryption config
resource "aws_s3_bucket_server_side_encryption_configuration" "sse" {
  bucket = aws_s3_bucket.bucket.id

  rule {
    apply_server_side_encryption_by_default {
      kms_master_key_id = var.kms_master_key_id
      sse_algorithm     = var.sse_algorithm
    }
  }
}

#permission to trigger lambda [created if [s3_events_lambda_enabled=true]
resource "aws_lambda_permission" "allow_bucket" {
  count         = length(var.lambda_arn) > 0 ? 1 : 0
  statement_id  = "AllowExecutionFromS3Bucket"
  action        = "lambda:InvokeFunction"
  function_name = var.lambda_arn
  principal     = "s3.amazonaws.com"
  source_arn    = aws_s3_bucket.bucket.arn
}

#sets objetc ownership to bucket owner full control if enable_bucket_owner_full_control_acl is set to true
resource "aws_s3_bucket_ownership_controls" "object_owner_config" {
  count      = var.enable_bucket_owner_full_control_acl ? 1 : 0
  depends_on = [aws_s3_bucket_public_access_block.bucket_access_block, aws_s3_bucket_notification.bucket_notification]
  bucket     = aws_s3_bucket.bucket.id
  rule {
    object_ownership = "BucketOwnerPreferred"
  }
}

/*bucket event notification
  sends a notification to lambda, sqs or sns
  whenever a new object is created
*/
#enabled if either of the three event settings (s3_events_lambda_enabled,s3_events_sqs_enabled,s3_events_sns_enabled) have been set to true
resource "aws_s3_bucket_notification" "bucket_notification" {
  count      = length(var.lambda_arn) > 0 || length(var.sqs_arn) > 0 || length(var.sns_arn) > 0 ? 1 : 0
  depends_on = [aws_s3_bucket_public_access_block.bucket_access_block, aws_lambda_permission.allow_bucket]
  bucket     = aws_s3_bucket.bucket.id

  #if s3 events for lambda is enabled
  dynamic "lambda_function" {
    for_each = length(var.lambda_arn) > 0 ? ["true"] : []
    content {
      lambda_function_arn = var.lambda_arn
      events              = local.events
      filter_prefix       = var.filter_prefix
      filter_suffix       = var.filter_suffix
    }
  }

  #if s3 events for sqs queue is enabled
  dynamic "queue" {
    for_each = length(var.sqs_arn) > 0 ? ["true"] : []
    content {
      queue_arn     = var.sqs_arn
      events        = local.events
      filter_prefix = var.filter_prefix
      filter_suffix = var.filter_suffix
    }
  }

  #if s3 events for sns topic is enabled
  dynamic "topic" {
    for_each = length(var.sns_arn) > 0 ? ["true"] : []
    content {
      topic_arn     = var.sns_arn
      events        = local.events
      filter_prefix = var.filter_prefix
      filter_suffix = var.filter_suffix
    }
  }
}

/* TODO:
check logging configuration and add it
*/

resource "aws_iam_policy" "read-only-policy" {
  count       = var.enable_iam_policies ? 1 : 0
  name        = "${var.iam_policy_prefix}-s3-read-only"
  path        = "/"
  description = "S3 read-only policy"

  policy = <<EOF
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Action": [
        "s3:Get*",
        "s3:List*"
      ],
      "Effect": "Allow",
      "Resource": ["${aws_s3_bucket.bucket.arn}", "${aws_s3_bucket.bucket.arn}/*"]
    }
  ]
}
EOF
  lifecycle {
    ignore_changes = [
      description,
    ]
  }

  tags = var.tags
}

resource "aws_iam_policy" "read-write-policy" {
  count       = var.enable_iam_policies ? 1 : 0
  name        = "${var.iam_policy_prefix}-s3-read-write"
  path        = "/"
  description = "S3 read and write objects policy"

  policy = <<EOF
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Action": [
        "s3:DeleteObject*",
        "s3:Get*",
        "s3:PutObject*",
        "s3:List*",
        "s3:AbortMultipartUpload"
      ],
      "Effect": "Allow",
      "Resource": ["${aws_s3_bucket.bucket.arn}", "${aws_s3_bucket.bucket.arn}/*"]
    }
  ]
}
EOF
  lifecycle {
    ignore_changes = [
      description,
    ]
  }

  tags = var.tags
}

resource "aws_iam_policy" "bucket-lifecycle-policy" {
  count       = var.enable_iam_policies ? 1 : 0
  name        = "${var.iam_policy_prefix}-s3-bucket-lifecycle"
  path        = "/"
  description = "S3 administrate bucket lifecycle policy"

  policy = <<EOF
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Action": [
        "s3:GetLifecycleConfiguration",
        "s3:PutLifecycleConfiguration"
      ],
      "Effect": "Allow",
      "Resource": ["${aws_s3_bucket.bucket.arn}", "${aws_s3_bucket.bucket.arn}/*"]
    }
  ]
}
EOF
  lifecycle {
    ignore_changes = [
      description,
    ]
  }

  tags = var.tags
}